@@ -12,6 +12,11 @@ sns.set_context('talk')
 #Image processing packages
 from matplotlib import image as mplImage
 import pytesseract
+if os.name == 'nt':
+    pytesseract.pytesseract.tesseract_cmd = r'C:\Program Files\Tesseract-OCR\tesseract.exe'
+    dirSep = '\\'
+else:
+    dirSep = '/'
 from PIL import Image
 from scipy import ndimage
 import cv2
@@ -24,7 +29,7 @@ from itertools import tee
 from tqdm.auto import trange
 from scipy.signal import argrelmin,find_peaks,savgol_filter
 import warnings
-import subprocess
+import shutil
 
 warnings.filterwarnings("ignore")
 
@@ -105,37 +110,51 @@ def returnColorScale(colorBar):
 
     return trueRGBColorScale
 
-def returnColorScaleSpan(legend,colorScale,colorBarScale):
-    croppedLegend = np.multiply(legend[60:105,:75],255)
-    pIm = Image.fromarray(np.uint8(croppedLegend))
-    fullString = pytesseract.image_to_string(pIm)
-    splitStrings = fullString.split('\n')
-    for splitString in splitStrings:
-        if 'Min' in splitString or 'Max' in splitString:
-          #Account for occasinoal pytesseract failures
-          if ' = ' in splitString:
-            splitChar = ' = '
-          else:
-            if ' =' in splitString:
-              splitChar = ' ='
-            elif '= ' in splitString:
-              splitChar = '= '
-            else:
-              if '=' in splitString:
-                splitChar = '='
+def returnColorScaleSpan(legend,colorScale,colorBarScale,cbar_lim=[]):
+    
+    #If using pytesseract to read in colorbar scale limits
+    if len(cbar_lim) == 0:
+        splitChar = True
+        croppedLegend = np.multiply(legend[60:105,:75],255)
+        pIm = Image.fromarray(np.uint8(croppedLegend))
+        fullString = pytesseract.image_to_string(pIm)
+        splitStrings = fullString.split('\n')
+        for splitString in splitStrings:
+            if 'Min' in splitString or 'Max' in splitString:
+              #Account for occasinoal pytesseract failures
+              if ' = ' in splitString:
+                splitChar = ' = '
               else:
-                print('PyTessearct error; equal sign not found. Split string is:'+splitString)
-          if 'Min' in splitString:
-              number = splitString.split(splitChar)[1]
-              minScalingFactor = 10**int(number[-1])
-              scaleStart = float(number[:4])*minScalingFactor
-          elif 'Max' in splitString:
-              number = splitString.split(splitChar)[1]
-              maxScalingFactor = 10**int(number[-1])
-              scaleEnd = float(number[:4])*maxScalingFactor
-            
-    linearScale = np.linspace(scaleStart,scaleEnd,num=len(colorScale))
-    return linearScale,scaleStart
+                if ' =' in splitString:
+                  splitChar = ' ='
+                elif '= ' in splitString:
+                  splitChar = '= '
+                else:
+                  if '=' in splitString:
+                    splitChar = '='
+                  else:
+                    splitChar = False
+              if splitChar:
+                  if 'Min' in splitString:
+                      number = splitString.split(splitChar)[1]
+                      minScalingFactor = 10**int(number[-1])
+                      scaleStart = float(number[:4])*minScalingFactor
+                      hasMin = True
+                  elif 'Max' in splitString:
+                      number = splitString.split(splitChar)[1]
+                      maxScalingFactor = 10**int(number[-1])
+                      scaleEnd = float(number[:4])*maxScalingFactor
+                      hasMax = True
+        if hasMin and hasMax:
+            linearScale = np.linspace(scaleStart,scaleEnd,num=len(colorScale))
+            return linearScale,scaleStart
+        else:
+            raise NameError('pytesseract automatic colorbar reading failed! Try setting a MANUAL axis limit instead')
+    else:
+        scaleStart = cbar_lim[0]
+        scaleEnd = cbar_lim[1]
+        linearScale = np.linspace(scaleStart,scaleEnd,num=len(colorScale))
+        return linearScale,scaleStart
 
 def returnLuminescentImageComponents(luminescent,visualize=False):
     occupancyCutoff = 300
@@ -330,6 +349,8 @@ def verticallySeparateMice(mouseBrightfieldMatrix,breakpoints,visualize=False):
             keptIntervals.append(interval)
 
     peaks,finalKeptIntervals = [],[]
+    
+    scaledInvervalValues = MinMaxScaler().fit_transform
     secondaryCutoff = 0.1
     for interval in keptIntervals:
         startpoint = interval[0]
@@ -351,6 +372,10 @@ def verticallySeparateMice(mouseBrightfieldMatrix,breakpoints,visualize=False):
                 break
         finalKeptIntervals.append([leftEndpoint,rightEndpoint])
     
+    #Scale peaks by image width to keep peaks tightly clustered between images
+    minPlot = min(verticalMouseSeparationDf.index.get_level_values('Column').tolist())
+    maxPlot = max(verticalMouseSeparationDf.index.get_level_values('Column').tolist())
+    peaks = [(x-minPlot)/(maxPlot-minPlot) for x in peaks]
     if visualize:
         verticalMouseSeparationDf.loc[:,:] = data.reshape(-1,1)
         g = sns.relplot(data=verticalMouseSeparationDf,x='Column',y='Count',kind='line')
@@ -536,13 +561,13 @@ def returnRadianceMetrics(imageTitle,samples,splitBrightfields,colorScale,linear
     
     return statisticDf
 
-def processGroupedMouseImage(imageTitle,luminescent,brightfield,brightfield2,visualize=False,save_pixel_df=False,sampleNames = [],save_images=False):
+def processGroupedMouseImage(imageTitle,luminescent,brightfield,brightfield2,visualize=False,save_pixel_df=False,sampleNames = [],save_images=False,cbar_lim=[]):
     
     #Preprocess images
     luminescentSamples,colorBar,legend,colorBarScale = returnLuminescentImageComponents(luminescent,visualize=visualize)
     
     colorScale = returnColorScale(colorBar)
-    linearScale,trueMin = returnColorScaleSpan(legend,colorScale,colorBarScale)
+    linearScale,trueMin = returnColorScaleSpan(legend,colorScale,colorBarScale,cbar_lim=cbar_lim)
     brightfieldSamples,originalBrightfieldSamples = rescaleBrightfieldImage(brightfield,brightfield2,luminescentSamples,visualize=visualize)
 
     #Crop images
@@ -600,7 +625,7 @@ def luminescentBrightfieldMatchCheck(sampleNameFile,save_pixel_df=False):
   if save_pixel_df:
     if 'imageMatrices' not in os.listdir(outputDir):
       os.mkdir(outputDir+'imageMatrices')    
-    
+
   for i in range(len(days)):
       day = days[i]
       tempDf = sampleNameFile.query("Day == @day")
@@ -621,13 +646,13 @@ def amendSampleNames(fullDf,allPeaks,sampleNameFile,fullSplitGroupDict,save_pixe
 
     clusterer = hdbscan.HDBSCAN(min_cluster_size=int(len(allPeaks)*0.1))
     cluster_labels = clusterer.fit_predict(np.array(allPeaks).reshape(-1,1))
-    cluster_labels = [str(x+1) for x in cluster_labels]  
+    cluster_labels = [str(x+1) for x in cluster_labels]
     tempDf = pd.DataFrame({'Max':allPeaks,'Cluster':cluster_labels})
-    sortedClusterList = tempDf.groupby(['Cluster']).mean().sort_values(by='Max').index.get_level_values('Cluster').tolist()
-    
+    sortedClusterList = tempDf.groupby(['Cluster']).median().sort_values(by='Max').index.get_level_values('Cluster').tolist()
+
     topX = 5
-    topXclusters = tempDf.groupby(['Cluster']).count().sort_values(by=['Max'],ascending=False).index.get_level_values('Cluster').tolist()[:topX]    
-    allClusterMeansDf = tempDf.groupby(['Cluster']).mean()
+    topXclusters = tempDf.groupby(['Cluster']).count().sort_values(by=['Max'],ascending=False).index.get_level_values('Cluster').tolist()[:topX]
+    allClusterMeansDf = tempDf.groupby(['Cluster']).median()
     clusterMeans = sorted([allClusterMeansDf.loc[x].values[0] for x in topXclusters])
     partitions = [0] + [(clusterMeans[x]+clusterMeans[x+1])/2 for x in range(len(clusterMeans)-1)] + [np.max(tempDf['Max'])]
     new_cluster_labels = []
@@ -637,15 +662,35 @@ def amendSampleNames(fullDf,allPeaks,sampleNameFile,fullSplitGroupDict,save_pixe
             if val > partitions[i] and val <= partitions[i+1]:
                 new_cluster_labels.append(str(i+1))
                 break
-    
+
+    tempDf['Cluster'] = new_cluster_labels
     #new_cluster_labels = [str(sortedClusterList.index(x)+1) for x in cluster_labels]
-    
+
     newFullDf = fullDf.copy()
     newFullDf.index.names = ['Position' if x == 'Sample' else x for x in fullDf.index.names]
     newFullDf = newFullDf.assign(Sample=new_cluster_labels).set_index('Sample', append=True)
-    
+    #newFullDf['Max'] = tempDf['Max']
+
+    #Make sure each 5 mouse group has 5 unique samples. If not, order by maxPeak
+    indexingDf = newFullDf.droplevel('Sample')
+    indexingDf = indexingDf.query("Position == '1'").droplevel('Position')
+    subsetDfList = []
+    allSplitGroupCombos = [[x.split(',,')[0],x.split(',,')[1][0]] for x in list(fullSplitGroupDict.keys())]
+    for row in range(indexingDf.shape[0]):
+        name = indexingDf.iloc[row,:].name
+        subsetDf = newFullDf.xs(name,level=tuple(indexingDf.index.names),drop_level=False).reset_index()
+        #Not a split group
+        if name[0:2] not in allSplitGroupCombos:
+            if subsetDf.shape[0] == 5 and len(list(pd.unique(subsetDf['Sample']))) != 5:
+                subsetDf['Sample'] = subsetDf['Position']
+        subsetDf = subsetDf.set_index(newFullDf.index.names)
+        subsetDfList.append(subsetDf)
+    #Combine sorted 5-mouse dfs (and others)
+    newFullDf = pd.concat(subsetDfList)
+
     fullDfList = []
     splitGroups = [',,'.join(x.split(',,')[:2]) for x in fullSplitGroupDict]
+    splitGroups = [x[:-1] for x in splitGroups]
     matrixRenamingDict = {}
     for currentDay in newFullDf.index.unique('Day'):
         dayDf = newFullDf.query("Day == @currentDay")
@@ -664,13 +709,14 @@ def amendSampleNames(fullDf,allPeaks,sampleNameFile,fullSplitGroupDict,save_pixe
                     splitPositionDict[key.split(',,')[2]] = 0
                 for key in keys:
                     splitKey = key.split(',,')
-                    day,group,samples = splitKey[0],splitKey[1],splitKey[2].split(',')
+                    day,originalGroup,samples = splitKey[0],splitKey[1],splitKey[2].split(',')
+                    group = originalGroup[:-1]
                     renamingPositions = fullSplitGroupDict[key]
                     subsetDf = newFullDf.query("Day == @day and Group == @group and Position == @samples").query("Sample == @renamingPositions")
                     for position,renamedPosition in zip(subsetDf.index.unique('Position'),renamingPositions):
                         splitPositionDict[splitKey[2]]+=1
                         trueIndex = newFullDf.query("Day == @day and Group == @group and Position == @samples").index.unique('Sample').tolist().index(renamedPosition)
-                        matrixRenamingDict['-'.join([currentDay,currentGroup,renamedPosition])] = '-'.join([currentDay,currentGroup+','.join(renamingPositions),str(trueIndex+1)])
+                        matrixRenamingDict['-'.join([currentDay,currentGroup,renamedPosition])] = '-'.join([currentDay,originalGroup+'_'+','.join(renamingPositions),str(trueIndex+1)])
                     subsetDfList.append(subsetDf)
                 renamedEntry = pd.concat(subsetDfList).sort_values(by=['Sample'])
             else:
@@ -688,29 +734,31 @@ def amendSampleNames(fullDf,allPeaks,sampleNameFile,fullSplitGroupDict,save_pixe
                     renamedEntry = renamedEntry.rename(renamingDict,level='Sample')
             renamedEntry = renamedEntry.sort_values(by=['Sample']).droplevel('Position')
             fullDfList.append(renamedEntry)
-    
+
     if save_pixel_df:
         savezDict,minScaleDict = {},{}
         for savezKey in matrixRenamingDict:
             oldFileName = matrixRenamingDict[savezKey]
-            savezDict[savezKey] = np.load(base_dir+'imageMatrices/'+oldFileName+'.npy')            
+            savezDict[savezKey] = np.load(base_dir+'imageMatrices/'+oldFileName+'.npy')
             minScaleDict[savezKey] = pickle.load(open(base_dir+'imageMatrices/'+oldFileName+'.pkl','rb'))
-        
+
         #Save concatenated files
-        experimentName = os.getcwd().split('/')[-1]
+        experimentName = os.getcwd().split(dirSep)[-1]
         np.savez_compressed(base_dir+experimentName+'-pixel',**savezDict)
         with open(base_dir+experimentName+'-minScale.pkl','wb') as f:
             pickle.dump(minScaleDict,f)
         #Delete temporary directory
         shutil.rmtree(base_dir+'imageMatrices/')
-        
-    fullDf = pd.concat(fullDfList)    
+
+    fullDf = pd.concat(fullDfList)
     return fullDf
 
-def checkSplitGroups(day,group,allPeaks,sampleNames=[],visualize=False,save_df=True,save_pixel_df=False,save_images=False):
+def checkSplitGroups(day,group,allPeaks,sampleNames=[],visualize=False,save_df=True,save_pixel_df=False,save_images=False,cbar_lim=[]):
     base_dir = 'inputData/'
+    #Check if length of image file name is > 1 (more than just a letter) -> split group
     luminescentImages = sorted([x.split('.')[0] for x in os.listdir(base_dir+'luminescent/'+day+'/') if len(x.split('.')[0]) > 1 and group in x.split('.')[0]])
     brightfieldImages = sorted([x.split('.')[0] for x in os.listdir(base_dir+'brightfield/'+day+'/') if len(x.split('.')[0]) > 1 and group in x.split('.')[0]])
+
     splitGroupDict = {}
     if len(luminescentImages) > 1:
         if set(luminescentImages) == set(brightfieldImages):
@@ -730,9 +778,9 @@ def checkSplitGroups(day,group,allPeaks,sampleNames=[],visualize=False,save_df=T
                 else:
                   fileName = base_dir+'brightfield/'+day+'/'+splitLuminescentFileName+'.TIF'
                 brightfield = mplImage.imread(fileName)
-                brightfield2 = cv2.imread(fileName)        
-                positionsToKeep = splitLuminescentFileName[1:].split(',')
-                groupDf,groupPeaks = processGroupedMouseImage(day+'-'+splitLuminescentFileName,luminescent,brightfield,brightfield2,sampleNames=sampleNames,visualize=visualize,save_images=save_images,save_pixel_df=save_pixel_df)
+                brightfield2 = cv2.imread(fileName)
+                positionsToKeep = splitLuminescentFileName[1:].split('_')[1].split(',')
+                groupDf,groupPeaks = processGroupedMouseImage(day+'-'+splitLuminescentFileName,luminescent,brightfield,brightfield2,sampleNames=sampleNames,visualize=visualize,save_images=save_images,save_pixel_df=save_pixel_df,cbar_lim=cbar_lim)
                 if splitIndex != 0:
                     originalSamples = groupDf.index.unique('Sample').tolist()
                     newSamples = [str(int(x)+splitIndex) for x in originalSamples]
@@ -740,13 +788,14 @@ def checkSplitGroups(day,group,allPeaks,sampleNames=[],visualize=False,save_df=T
                     for og,new in zip(originalSamples,newSamples):
                         renamingDict[og] = new
                     groupDf = groupDf.rename(renamingDict,level='Sample')
-                
+
                 splitIndex+=len(groupDf.index.unique('Sample').tolist())
-                splitGroupDict[day+',,'+group+',,'+','.join(groupDf.index.unique('Sample').tolist())] = positionsToKeep
+                splitGroupDict[day+',,'+splitLuminescentFileName.split('_')[0]+',,'+','.join(groupDf.index.unique('Sample').tolist())] = positionsToKeep
+                #splitGroupDict[day+',,'+group+',,'+','.join(groupDf.index.unique('Sample').tolist())] = positionsToKeep
                 groupDfList.append(groupDf)
                 groupPeaksList+=groupPeaks
             groupDf = pd.concat(groupDfList)
-            return groupDf,groupPeaksList,splitGroupDict           
+            return groupDf,groupPeaksList,splitGroupDict
         else:
             print('These images are not shared:\n')
             for missingImage in list(set(luminescentImages) ^ set(brightfieldImages)):
@@ -768,7 +817,7 @@ def checkSplitGroups(day,group,allPeaks,sampleNames=[],visualize=False,save_df=T
           fileName = base_dir+'brightfield/'+day+'/'+group+'.TIF'
         brightfield = mplImage.imread(fileName)
         brightfield2 = cv2.imread(fileName)        
-        groupDf,groupPeaks = processGroupedMouseImage(day+'-'+group,luminescent,brightfield,brightfield2,sampleNames=sampleNames,visualize=visualize,save_images=save_images,save_pixel_df=save_pixel_df)
+        groupDf,groupPeaks = processGroupedMouseImage(day+'-'+group,luminescent,brightfield,brightfield2,sampleNames=sampleNames,visualize=visualize,save_images=save_images,save_pixel_df=save_pixel_df,cbar_lim=cbar_lim)
         return groupDf,groupPeaks,splitGroupDict
 
 def moveRawImages(sampleNameFile,pathToRawImages):
@@ -778,7 +827,7 @@ def moveRawImages(sampleNameFile,pathToRawImages):
         if imageType not in os.listdir('inputData'):
             os.mkdir('inputData/'+imageType)
         for day in list(pd.unique(sampleNameFile['Day'])):
-            newDay = 'D'+''.join([i for i in day.split() if i.isdigit()])
+            newDay = 'D'+''.join([i for i in day if i.isdigit()])
             dayRenamingDict[day] = newDay
             if day in os.listdir(pathToRawImages):
                 if newDay not in os.listdir('inputData/'+imageType):
@@ -795,14 +844,15 @@ def moveRawImages(sampleNameFile,pathToRawImages):
                                     break
                         finalPath =  'inputData/'+imageType+'/'+newDay+'/'
                         finalName = group+fileExtensionDict[imageType]
-                        subprocess.run(['cp',initialPath+initialName,finalPath+finalName])
+                        shutil.copyfile(initialPath+initialName,finalPath+finalName)
+    
     dayIndex = list(sampleNameFile.columns).index('Day')
     for i in range(sampleNameFile.shape[0]):
         oldDay = sampleNameFile.iloc[i,dayIndex]
         sampleNameFile.iloc[i,dayIndex] = dayRenamingDict[oldDay]
     return sampleNameFile
 
-def fullInVivoImageProcessingPipeline(sampleNameFile,visualize=False,save_df=True,save_pixel_df=False,save_images=False,pathToRawImages=''):
+def fullInVivoImageProcessingPipeline(sampleNameFile,visualize=False,save_df=True,save_pixel_df=False,save_images=False,pathToRawImages='',cbar_lim=[]):
   outputDir = 'outputData/'
   if pathToRawImages != '':
       sampleNameFile = moveRawImages(sampleNameFile,pathToRawImages)
@@ -822,7 +872,7 @@ def fullInVivoImageProcessingPipeline(sampleNameFile,visualize=False,save_df=Tru
             group = groups[j]
             sampleNames = []
             
-            groupDf,groupPeaks,splitGroupDict = checkSplitGroups(day,group,dayPeaksList,sampleNames=sampleNames,visualize=visualize,save_images=save_images,save_pixel_df=save_pixel_df)
+            groupDf,groupPeaks,splitGroupDict = checkSplitGroups(day,group,dayPeaksList,sampleNames=sampleNames,visualize=visualize,save_images=save_images,save_pixel_df=save_pixel_df,cbar_lim=cbar_lim)
             groupDfList.append(groupDf)
             dayPeaksList+=groupPeaks
             fullSplitGroupDict = {**fullSplitGroupDict,**splitGroupDict}
@@ -830,7 +880,7 @@ def fullInVivoImageProcessingPipeline(sampleNameFile,visualize=False,save_df=Tru
         dayDf = pd.concat(groupDfList,keys=groups,names=['Group'])
         dayDfList.append(dayDf)
 
-    experimentName = os.getcwd().split('/')[-1]
+    experimentName = os.getcwd().split(dirSep)[-1]
     outputFileName = 'radianceStatisticPickleFile-'+experimentName
     if 'SampleNames' not in tempDf.columns:
         sampleNamesColumn = []
