@@ -10,6 +10,10 @@ import radianceQuantifier.dataprocessing.miscFunctions as mf
 import radianceQuantifier.plotting.facetPlotLibrary as fpl 
 import radianceQuantifier.plotting.interactiveGUIElements as ipe
 
+if os.name == 'nt':
+    dirSep = '\\'
+else:
+    dirSep = '/'
 expParamDict = {'radiance':'cell'}
 
 #Get level names and values into an easily accessible dictionary
@@ -386,8 +390,8 @@ class MouseImageSelectionPage(tk.Frame):
             templatePathDict = pickle.load(open(master.homedirectory + 'misc/templatePathDict.pkl', 'rb'))
         else:
             templatePathDict = {}
-        projectName = os.getcwd().split('/')[(-2)]
-        experimentName = os.getcwd().split('/')[(-1)]
+        projectName = os.getcwd().split(dirSep)[(-2)]
+        experimentName = os.getcwd().split(dirSep)[(-1)]
         templatePath = templatePathDict[projectName + '/' + experimentName]
         global sampleNameFile
         if '.csv' in templatePath:
@@ -395,10 +399,17 @@ class MouseImageSelectionPage(tk.Frame):
         else:
             sampleNameFile = pd.read_excel(templatePath)
         
+        for i in range(sampleNameFile.shape[0]):
+            for j in range(sampleNameFile.shape[1]):
+                if not pd.isna(sampleNameFile.iloc[i,j]):
+                    sampleNameFile.iloc[i,j] = str(sampleNameFile.iloc[i,j])
+                    if sampleNameFile.iloc[i,j].replace(".", "",1).isdigit():
+                        sampleNameFile.iloc[i,j] = sampleNameFile.iloc[i,j].rstrip("0").rstrip(".")
+
         #Ensure day names are compatible
         dayRenamingDict = {}
         for day in list(pd.unique(sampleNameFile['Day'])):
-            newDay = 'D'+''.join([i for i in day.split() if i.isdigit()])
+            newDay = 'D'+''.join([i for i in day if i.isdigit()])
             dayRenamingDict[day] = newDay
         dayIndex = list(sampleNameFile.columns).index('Day')
         for i in range(sampleNameFile.shape[0]):
@@ -406,7 +417,9 @@ class MouseImageSelectionPage(tk.Frame):
             sampleNameFile.iloc[i,dayIndex] = dayRenamingDict[oldDay]
         
         days = pd.unique(sampleNameFile['Day']).tolist()
+        days = sorted(days, key=lambda x: int(''.join([s for s in x if s.isdigit()])))
         groups = pd.unique(sampleNameFile['Group']).tolist()
+        groups = sorted(groups)
 
         dayWindow = tk.Frame(mainWindow)
         contentWindow = tk.Frame(dayWindow)
@@ -522,18 +535,24 @@ class MouseImagePlottingOptionsPage(tk.Frame):
         cmapEntry.grid(row=0,column=1,sticky=tk.W)
         cmapEntry.set(defaultCmaps[0])
 
-        tk.Label(mainWindow,text='Font size:').grid(row=1,column=0,sticky=tk.W)
-        fontSizeEntry = tk.Entry(mainWindow)
-        fontSizeEntry.grid(row=1,column=1,sticky=tk.W)
-        fontSizeEntry.insert(tk.END, '42')
+        tk.Label(mainWindow,text='Font Scale:').grid(row=1,column=0,sticky=tk.W)
+        fontScaleEntry = tk.Entry(mainWindow)
+        fontScaleEntry.grid(row=1,column=1,sticky=tk.W)
+        fontScaleEntry.insert(tk.END, '1')
         
         tk.Label(mainWindow,text='Plot Title:').grid(row=2,column=0,sticky=tk.W)
         titleEntry = tk.Entry(mainWindow)
         titleEntry.grid(row=2,column=1,sticky=tk.W)
         titleEntry.insert(tk.END, selectionTitle)
         
+        tailCropVar = tk.BooleanVar()
+        tailCropCB = tk.Checkbutton(mainWindow,text='Crop tail',variable=tailCropVar)
+        tailCropCB.select()
+        tailCropCB.grid(row=3,column=0,columnspan=2)
+        
         def createPlot():
-            plotMouseImages(subsetMatrix,minScaleDict,selectionKeysDf,innerCol='Sample',row='Day',col='Group',cmap=cmapEntry.get(),save_image=True,imageTitle=titleEntry.get(),fontsize=int(fontSizeEntry.get()),groupRenamingDict=groupRenamingDict)
+            maxTextLength = len(max(list(groupRenamingDict.values()),key=len))
+            plotMouseImages(subsetMatrix,minScaleDict,selectionKeysDf,tailCrop=tailCropVar.get(),innerCol='Sample',row='Day',col='Group',cmap=cmapEntry.get(),save_image=True,imageTitle=titleEntry.get(),fontScale=int(fontScaleEntry.get()),groupRenamingDict=groupRenamingDict,maxTextLength=maxTextLength)
             tk.messagebox.showinfo(title='Success', message='Plot created!')
             self.FinishButton.config(state=tk.NORMAL)
 
@@ -979,6 +998,18 @@ class plotElementsGUIPage(tk.Frame):
                 else:
                     plotSpecificDict['scaleToMode'] = False
                 plotSpecificDict['smoothing'] = int(ipe.getSliderValues(smoothingSliderList,['smoothing'])['smoothing'])
+            elif subPlotType == 'line':
+                ciType = ipe.getRadiobuttonValues(ciSelectionRadiobuttonVarsDict)['Error bar']
+                errType = ipe.getRadiobuttonValues(ciStyleSelectionRadiobuttonVarsDict)['Error bar style']
+                if ciType == 'st dev':
+                    plotSpecificDict['ci'] = 'sd'
+                elif ciType == 'conf int':
+                    plotSpecificDict['ci'] = 95
+
+                if errType == 'band' and ciType in ['st dev','conf int']:
+                    plotSpecificDict['err_style'] = 'band'
+                elif errType == 'bar' and ciType in ['st dev','conf int']:
+                    plotSpecificDict['err_style'] = 'bars'
             
             useModifiedDf = False
             sName = titleEntry.get()
@@ -1058,7 +1089,23 @@ class plotElementsGUIPage(tk.Frame):
             smoothnessWindow = tk.Frame(subPlotSpecificWindow)
             smoothnessWindow.grid(row=0,column=1,sticky=tk.N)
             smoothingSliderList = ipe.createParameterAdjustmentSliders(smoothnessWindow,['smoothing'],{'smoothing':[1,99,2,27]})
-        
+        elif subPlotType in ['line']:
+            #Scale to mode button
+            subPlotSpecificWindow = tk.Frame(self)
+            subPlotSpecificWindow.pack(side=tk.TOP,pady=10)
+
+            ciSelectionWindow = tk.Frame(subPlotSpecificWindow)
+            ciSelectionWindow.grid(row=0,column=0,sticky=tk.N)
+            ciSelectionRadiobuttonList,ciSelectionRadiobuttonVarsDict = ipe.createParameterSelectionRadiobuttons(ciSelectionWindow,['Error bar'],{'Error bar':['none','st dev','conf int']})
+            ciSelectionRadiobuttonVarsDict['Error bar'].set('none')
+
+            ciStyleSelectionWindow = tk.Frame(subPlotSpecificWindow)
+            ciStyleSelectionWindow.grid(row=0,column=1,sticky=tk.N)
+            ciStyleSelectionRadiobuttonList,ciStyleSelectionRadiobuttonVarsDict = ipe.createParameterSelectionRadiobuttons(ciStyleSelectionWindow,['Error bar style'],{'Error bar style':['bar','band']})
+            ciStyleSelectionRadiobuttonVarsDict['Error bar style'].set('bar')
+
+
+
         plotButtonWindow = tk.Frame(self)
         plotButtonWindow.pack(side=tk.TOP,pady=10)
         tk.Button(plotButtonWindow, text="Generate First Plot",command=lambda: collectInputs(False)).grid(row=0,column=0)
